import Cookies from 'js-cookie'
import storage from 'store';
import {APP_LANGUAGE} from '@/store/mutation-types';
import {loadLanguageAsync} from '@/locales';

const state = {
    sidebar: {
        opened: Cookies.get('sidebarStatus')
            ? !!+Cookies.get('sidebarStatus')
            : true,
        withoutAnimation: false
    },
    device: 'desktop',
    language: 'zh',
    size: 'mini', //default medium small mini Cookies.get('size')
    lang: JSON.parse(localStorage.getItem('lang')) || 'zh-CN',
    _antLocale: {}
}

const mutations = {
    TOGGLE_SIDEBAR: state => {
        state.sidebar.opened = !state.sidebar.opened
        state.sidebar.withoutAnimation = false
        if (state.sidebar.opened) {
            Cookies.set('sidebarStatus', 1)
        } else {
            Cookies.set('sidebarStatus', 0)
        }
    },
    CLOSE_SIDEBAR: (state, withoutAnimation) => {
        Cookies.set('sidebarStatus', 0)
        state.sidebar.opened = false
        state.sidebar.withoutAnimation = withoutAnimation
    },
    TOGGLE_DEVICE: (state, device) => {
        state.device = device
    },
    SET_LANGUAGE: (state, language) => {
        state.language = language
        Cookies.set('language', language)
    },
    SET_SIZE: (state, size) => {
        state.size = size
        Cookies.set('size', size)
    },
    [APP_LANGUAGE]: (state, lang, antd = {}) => {
        state.lang = lang;
        state._antLocale = antd;
        storage.set(APP_LANGUAGE, lang);
    }
}

const actions = {
    toggleSideBar({commit}) {
        commit('TOGGLE_SIDEBAR')
    },
    closeSideBar({commit}, {withoutAnimation}) {
        commit('CLOSE_SIDEBAR', withoutAnimation)
    },
    toggleDevice({commit}, device) {
        commit('TOGGLE_DEVICE', device)
    },
    setLanguage({commit}, language) {
        commit('SET_LANGUAGE', language)
    },
    setSize({commit}, size) {
        commit('SET_SIZE', size)
    },
    setLang({commit}, lang) {
        return new Promise((resolve, reject) => {
            commit(APP_LANGUAGE, lang);
            loadLanguageAsync(lang).then(() => {
                resolve();
            }).catch((e) => {
                reject(e);
            });
        });
    }
}

export default {
    namespaced: true,
    state,
    mutations,
    actions
}
