import { login, logout, getInfo, loginByWechat } from '@/api/user'
import { getToken, setToken, removeToken } from '@/utils/auth'
import router, { resetRouter } from '@/router'

const state = {
  token: getToken(),
  name: '',
  avatar: '',
  introduction: '',
  roles: [],
  authType: '',
  userInfo: null,
  user: null,
  userId: null
}

const mutations = {
  SET_TOKEN: (state, token) => {
    state.token = token
  },
  SET_INTRODUCTION: (state, introduction) => {
    state.introduction = introduction
  },
  SET_NAME: (state, name) => {
    state.name = name
  },
  SET_AVATAR: (state, avatar) => {
    state.avatar = avatar
  },
  SET_ROLES: (state, roles) => {
    state.roles = roles
  },
  SET_AUTH_TYPE: (state, authType) => {
    state.authType = authType
  },
  SET_USERINFO: (state, userInfo) => {
    state.userInfo = userInfo
  },
  SET_USERID: (state, userId) => {
    state.userId = userId
  }
}

const actions = {
  setAuthType({ commit }, authType) {
    commit('SET_AUTH_TYPE', authType)
  },
  LoginByWechat({ commit }, wechatInfo) {
    const { code, state } = wechatInfo
    return new Promise((resolve, reject) => {
      loginByWechat({
        code: code,
        invitationCode: state && state != 'undefined' ? state : ''
      })
        .then(response => {
          if (response.status === 0) {
            const { data } = response
            commit('SET_TOKEN', data.token.token)
            setToken(data.token.token)
          }
          resolve(response)
        })
        .catch(error => {
          reject(error)
        })
    })
  },
  // user login
  login({ commit }, userInfo) {
    const { username, password } = userInfo
    return new Promise((resolve, reject) => {
      login({ userName: username.trim(), password: password })
        .then(response => {
          const { data } = response
          commit('SET_USERID', data.user.userId)
          commit('SET_TOKEN', data.token.token)
          setToken(data.token.token)
          resolve()
        })
        .catch(error => {
          reject(error)
        })
    })
  },

  // get user info
  getInfo({ commit, state }) {
    return new Promise((resolve, reject) => {
      getInfo()
        .then(response => {
          var { data } = response
          if (!data) {
            reject('Verification failed, please Login again.')
          }
          var { roles, name, avatar } = data

          if (!roles || roles.length <= 0) {
            roles = ['visitor']
            data.roles = roles
            // reject('getInfo: roles must be a non-null array!')
          }

          commit('SET_ROLES', roles ? roles : ['visitor'])
          commit('SET_NAME', name)
          commit('SET_AVATAR', avatar)
          commit('SET_USERINFO', data)
          commit('SET_USERID', data.userId)
          // commit('SET_INTRODUCTION', introduction)
          resolve(data)
        })
        .catch(error => {
          reject(error)
        })
    })
  },

  // user logout
  logout({ commit, state, dispatch }) {
    return new Promise((resolve, reject) => {
      logout(state.token)
        .then(() => {
          commit('SET_TOKEN', '')
          commit('SET_ROLES', [])
          commit('SET_USERID', null)
          commit('SET_USERINFO', null)
          removeToken()
          resetRouter()

          // reset visited views and cached views
          // to fixed https://github.com/PanJiaChen/vue-element-admin/issues/2485
          dispatch('tagsView/delAllViews', null, { root: true })

          resolve()
        })
        .catch(error => {
          reject(error)
        })
    })
  },

  // remove token
  resetToken({ commit }) {
    return new Promise(resolve => {
      commit('SET_TOKEN', '')
      commit('SET_ROLES', [])
      commit('SET_USERID', null)
      commit('SET_USERINFO', null)
      removeToken()
      resolve()
    })
  },

  // dynamically modify permissions
  async changeRoles({ commit, dispatch }, role) {
    const token = role + '-token'

    commit('SET_TOKEN', token)
    setToken(token)

    const { roles } = await dispatch('getInfo')

    resetRouter()

    // generate accessible routes map based on roles
    const accessRoutes = await dispatch('permission/generateRoutes', roles, {
      root: true
    })
    // dynamically add accessible routes
    router.addRoutes(accessRoutes)

    // reset visited views and cached views
    dispatch('tagsView/delAllViews', null, { root: true })
  }
}

export default {
  namespaced: true,
  state,
  mutations,
  actions
}
